/* 
 * File:   main.c
 * Author: David Meyer
 *
 * Created on June 13, 2012, 9:14 PM
 */

// DEVCFG3:
#pragma config IOL1WAY  = OFF       // Peripheral Pin Select Configuration, allow mult reconfig
#pragma config PMDL1WAY = OFF       // Peripheral Module Disable Config, allow mult reconfig
// DEVCFG2:
#pragma config UPLLEN   = ON        // USB PLL Enable
#pragma config UPLLIDIV = DIV_5     // USB PLL Divider
// 20MHz / 5 * 24 / 2 = 48MHz
#pragma config FPLLODIV = DIV_2     // PLL Output Divider
#pragma config FPLLMUL  = MUL_20    // PLL Multiplier
#pragma config FPLLIDIV = DIV_5     // PLL Input Divider
// 20MHz / 2 * 20 / 2 = 40MHz

// DEVCFG1:
#pragma config FWDTEN   = OFF       // Watchdog Timer
#pragma config WDTPS    = PS1       // Watchdog Timer Postscale
#pragma config FCKSM    = CSDCMD    // Clock Switching & Fail Safe Clock Monitor
#pragma config FPBDIV   = DIV_1     // Peripheral Clock divisor
#pragma config OSCIOFNC = OFF       // CLKO Enable
#pragma config POSCMOD  = EC        // Primary Oscillator
#pragma config IESO     = OFF       // Internal/External Switch-over
#pragma config FSOSCEN  = OFF       // Secondary Oscillator Enable (KLO was off)
#pragma config FNOSC    = PRIPLL    // Oscillator Selection
// DEVCFG0:
#pragma config CP       = OFF       // Code Protect
#pragma config BWP      = ON        // Boot Flash Write Protect
#pragma config PWP      = OFF       // Program Flash Write Protect
#pragma config ICESEL   = ICS_PGx1  // ICE/ICD Comm Channel Select
#pragma config JTAGEN   = OFF       // JTAG Enable
#pragma config DEBUG    = OFF       // Background Debugger Enable


/** Includes *******************************************************/
#include <plib.h>
#include "HardwareProfile.h"
#include "./USB/usb.h"
#include "./USB/usb_function_hid.h"



/** PreCompiler Definitions ****************************************/
#define mLED        LATBbits.LATB7
#define SW          PORTAbits.RA4
#define FastSwitch  LATBbits.LATB8


/** Global Variables ***********************************************/
USB_HANDLE USBOutHandle = 0;
USB_HANDLE USBInHandle = 0;
unsigned char ReceivedDataBuffer[64];
unsigned char ToSendDataBuffer[64];
char enumerated = 0;

int slider = 0;
int freq = 0;


/** Function Headers ***********************************************/


int main() {

    CFGCONbits.JTAGEN = 0; // turn off JTAG, get back those pins for IO use
    // set PIC32to max computing power

    // enable multi-vector interrupts
    INTEnableSystemMultiVectoredInt();
    PIC32MX250_setup_pins();
    SYSTEMConfigPerformance(SYS_FREQ);

    // define setup parameters for OpenADC10
    // Turn module on | output in integer | trigger mode auto | enable  autosample
    #define PARAM1  ADC_MODULE_ON | ADC_FORMAT_INTG | ADC_CLK_AUTO | ADC_AUTO_SAMPLING_ON

    // define setup parameters for OpenADC10
    // ADC ref external    | disable offset test    | enable scan mode | perform 1 samples | use one buffer | use MUXA mode
    #define PARAM2  ADC_VREF_AVDD_AVSS | ADC_OFFSET_CAL_DISABLE | ADC_SCAN_ON | ADC_SAMPLES_PER_INT_1 | ADC_ALT_BUF_OFF | ADC_ALT_INPUT_OFF
    // define setup parameters for OpenADC10
    // 				  use ADC internal clock | set sample time
    #define PARAM3  ADC_CONV_CLK_INTERNAL_RC | ADC_SAMPLE_TIME_15

    // define setup parameters for OpenADC10
    #define PARAM4	ENABLE_AN5_ANA

    // define setup parameters for OpenADC10
    // assign channels you don't want to scan (so list all of the pins that are not in PARAM4
    #define PARAM5  SKIP_SCAN_AN0 | SKIP_SCAN_AN1 | SKIP_SCAN_AN2 | SKIP_SCAN_AN3 | SKIP_SCAN_AN4 | SKIP_SCAN_AN6 | SKIP_SCAN_AN7 | SKIP_SCAN_AN8 | SKIP_SCAN_AN9 | SKIP_SCAN_AN10 | SKIP_SCAN_AN11 | SKIP_SCAN_AN12 | SKIP_SCAN_AN13 | SKIP_SCAN_AN14 | SKIP_SCAN_AN15

    // use ground as neg ref
    SetChanADC10(ADC_CH0_NEG_SAMPLEA_NVREF );
    OpenADC10( PARAM1, PARAM2, PARAM3, PARAM4, PARAM5 ); // configure ADC using parameter define above

    // Enable the ADC
    EnableADC10();

    // wait for the first conversion to complete so there will be valid data in ADC result registers
    while ( ! mAD1GetIntFlag() );
    

    USBDeviceInit();
    mLED = 0;

#ifdef USB_INTERRUPT
    USBDeviceAttach();
#endif


    OpenTimer3(T3_ON | T3_SOURCE_INT| T3_PS_1_256, 62500); // 1000 Hz PWM: 40000000 Hz / 64ps / 625 = 1000 Hz
    OpenOC3(OC_ON | OC_TIMER3_SRC | OC_PWM_FAULT_PIN_DISABLE, 0, 0);
    SetDCOC3PWM(6250);


    OpenTimer2(T2_ON | T2_SOURCE_INT | T2_PS_1_256, 15625); //  5 Hz PWM: 40000000 Hz / 256ps / 15625 = 5 Hz
    OpenOC2(OC_ON | OC_TIMER2_SRC | OC_PWM_FAULT_PIN_DISABLE, 0, 0);
    SetDCOC2PWM(2000);

    while(1){
#ifdef USB_POLLING
    USBDeviceTasks();
#endif

    FastSwitch = !FastSwitch;

    if((USBGetDeviceState() < CONFIGURED_STATE) ||
       (USBIsDeviceSuspended() == TRUE))
    {
        //Either the device is not configured or we are suspended,
        // so we don't want to execute any USB related application code
        SetDCOC2PWM(2000);
        enumerated = 0;
        continue;   //go back to the top of the while loop
    }
    if(!enumerated){
        SetDCOC2PWM(15625);
        enumerated = 1;
    }

    if(!HIDRxHandleBusy(USBOutHandle))				//Check if data was received from the host.
    {
        // Do something with the input data
        freq = 62500-((unsigned int)ReceivedDataBuffer[0])*61875/255;
        WritePeriod3(freq);
        SetDCOC3PWM(freq/10);

        mLED = ReceivedDataBuffer[1];

        //Re-arm the OUT endpoint for the next packet
        USBOutHandle = HIDRxPacket(HID_EP,(BYTE*)&ReceivedDataBuffer,64);
    }


    // Read in slider and switch and send to computer

    slider = ReadADC10(0);
    ToSendDataBuffer[0] = (unsigned char)!SW;
    ToSendDataBuffer[1] = slider/4;
    

    if(!HIDTxHandleBusy(USBInHandle)){
        // Send out new data
        USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer[0],64);
    }


    }

}


/** Interrupts *******************************************************/
